package responses

import (
	"html/template"
	"net/http"

	"git.kirsle.net/apps/gophertype/pkg/console"
)

// RenderTemplate renders a Go HTML template.
// The io.Writer can be an http.ResponseWriter.
func RenderTemplate(w http.ResponseWriter, r *http.Request, tmpl string, vars interface{}) error {
	if vars == nil {
		vars = NewTemplateVars(w, r)
	}

	// Look for the built-in template.
	b, err := GetFile(tmpl)
	if err == nil {
		t, err := template.New(tmpl).Funcs(TemplateFuncs(r)).Parse(string(b))
		if err != nil {
			console.Error("RenderTemplate: bundled template '%s': %s", tmpl, err)
			return err
		}

		// We found the template. Can we find the layout html?
		if layout, err := GetFile(".layout.gohtml"); err == nil {
			_, err := t.New("layout").Parse(string(layout))
			if err != nil {
				console.Error("RenderTemplate(.layout.gohtml): %s", err)
			}
		} else {
			console.Error("RenderTemplate: .layout.gohtml not found to wrap %s", tmpl)
		}

		console.Debug("Render Templ: %s", tmpl)
		if err := t.ExecuteTemplate(w, "layout", vars); err != nil {
			console.Error("RenderTemplate(%s): %s", tmpl, err)
		}
		return nil
	}

	Panic(w, http.StatusInternalServerError, err.Error())
	return nil
}
